"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zhin_1 = require("zhin");
require("@zhinjs/plugin-http-server");
const client_1 = require("./client");
(0, zhin_1.defineMetadata)({ name: 'com wechat adapter' });
const message_1 = require("./message");
const adapter = (0, zhin_1.registerAdapter)('com-wechat');
adapter.schema({
    type: zhin_1.Schema.const('ws', '连接方式'),
    url: zhin_1.Schema.string('请输入服务端ws地址'),
    access_token: zhin_1.Schema.string('请输入access_token'),
    max_reconnect_count: zhin_1.Schema.number('请输入max_reconnect_count').default(10),
    reconnect_interval: zhin_1.Schema.number('请输入reconnect_interval').default(3000),
});
class WechatClient extends zhin_1.Adapter.Bot {
    constructor(config) {
        super(adapter, config.unique_id, new client_1.Client(adapter, config, adapter.app.router));
    }
    async handleSendMessage(channel, message, source) {
        const [target_type, ...other] = channel.split(':');
        const target_id = other.join(':');
        const msg = message_1.Message.formatSegments(message);
        switch (target_type) {
            case 'group':
                return this.sendGroupMsg(target_id, msg);
            case 'private':
                return this.sendPrivateMsg(target_id, msg);
            default:
                throw new Error(`com wechat 适配器暂不支持发送${target_type}类型的消息`);
        }
    }
}
const startBots = (configs) => {
    if (!adapter.app?.server)
        throw new Error('“com-wechat miss require service “http”, maybe you need install “ @zhinjs/plugin-http-server ”');
    for (const config of configs) {
        const bot = new WechatClient(config);
        bot.on('message', messageHandler.bind(global, bot));
        bot.start().then(() => {
            adapter.emit('bot-ready', bot);
        });
        adapter.bots.push(bot);
    }
};
const messageHandler = (bot, event) => {
    const master = bot.config?.master;
    const admins = bot.config.admins?.filter(Boolean) || [];
    const message = zhin_1.Message.from(adapter, bot, {
        channel: `${event.detail_type}:${event.user_id}`,
        sender: {
            user_id: event.user_id,
            user_name: event.nickname || '',
            permissions: [
                master && event.user_id === master && 'master',
                admins && admins.includes(event.user_id) && 'admins',
                ...(event.permissions || []),
            ].filter(Boolean),
        },
        raw_message: message_1.Message.formatToString(event.message),
        message_type: event.detail_type,
    });
    adapter.app.emit('message', adapter, bot, message);
};
const stopBots = () => {
    for (const bot of adapter.bots) {
        bot.internal.stop();
    }
};
adapter.on('start', startBots);
adapter.on('stop', stopBots);
